// Popovers and hints

// Fix popovers in math equations
function addHints() {
  // console.log('Called...')

  $('.mathpopup').each(function (index) {
    var anchor = $(this).parent();
    var href = anchor.attr('href') // this contains the hint!

    if (href) {
      // dump when debugging...
      // console.log('Found one!')
      // console.log(href);

      anchor.removeAttr('href')
      anchor.addClass('fix-popover')
      anchor.attr('tabindex', '0')
      anchor.attr('role', 'button')
      anchor.attr('data-bs-trigger', 'focus')
      anchor.attr('data-bs-container', 'body')
      anchor.attr('data-bs-toggle', 'popover')
      anchor.attr('data-bs-html', 'true')
      anchor.attr('data-bs-placement', 'top')
      anchor.attr('data-bs-content', href)
    }
  });

  $('.fix-popover').each(function () {
    new bootstrap.Popover($(this));
    $(this).on('shown.bs.popover', function () {
      MathJax.typeset();
    });
  });

  $('.fix-popover').removeClass('fix-popover');
  $('.mathpopup').removeClass('mathpopup');
}


// Bootstrap tooltips
//
// see https://getbootstrap.com/docs/5.1/components/tooltips/#example-enable-tooltips-everywhere

var tooltipTriggerList = [].slice.call(document.querySelectorAll('[data-bs-toggle="tooltip"]'))
var tooltipList = tooltipTriggerList.map(function (tooltipTriggerEl) {
  return new bootstrap.Tooltip(tooltipTriggerEl)
})

// AnchorJS

anchors.options.placement = 'right';
anchors.add('.anchored');

// Popovers
//
// see https://getbootstrap.com/docs/5.1/components/popovers/#example-enable-popovers-everywhere

var popoverTriggerList = [].slice.call(document.querySelectorAll('[data-bs-toggle="popover"]'))
var popoverList = popoverTriggerList.map(function (popoverTriggerEl) {
  return new bootstrap.Popover(popoverTriggerEl, {
    html: true,
    sanitize: false,
    container: 'body',
    trigger: 'focus',
    content: function () {
      var content = $(popoverTriggerEl).attr("data-content-id");
      return $(content).html();
    }
  })
})

// Typeset possible math content in popovers

$('[data-bs-toggle="popover"]').on('shown.bs.popover', function () {
  MathJax.typeset();
});

// Settings
var settings = JSON.parse(localStorage.getItem('settings')) || {
  currentTheme: 'light',
  systemTheme: true,
  hideContents: false,
  bw: false,
  font: 'noto',
  fontSize: '1rem',
  lineHeight: '1.5',
}

window.addEventListener('storage', function (event) {
  if (event.key === 'settings') {
    settings = JSON.parse(localStorage.getItem('settings'))
    applySettings()
  }
})

document.getElementById('settings-modal').addEventListener('show.bs.modal', function () {
  document.getElementById('settings-switch-system-theme').checked = settings.systemTheme
  document.getElementById('settings-switch-hide-contents').checked = settings.hideContents
  document.getElementById('settings-switch-bw').checked = settings.bw
  document.getElementById('settings-select-font').value = settings.font || 'noto'
  document.getElementById('settings-select-font-size').value = settings.fontSize || '1rem'
  document.getElementById('settings-select-line-height').value = settings.lineHeight || '1.5'
})

document.getElementById('settings-save').addEventListener('click', function () {
  settings = {
    currentTheme: settings.currentTheme,
    systemTheme: document.getElementById('settings-switch-system-theme').checked,
    hideContents: document.getElementById('settings-switch-hide-contents').checked,
    bw: document.getElementById('settings-switch-bw').checked,
    font: document.getElementById('settings-select-font').value,
    fontSize: document.getElementById('settings-select-font-size').value,
    lineHeight: document.getElementById('settings-select-line-height').value,
  }
  if (settings.systemTheme) {
    settings.currentTheme = isSystemDark() ? 'dark' : 'light'
  }
  saveSettings()
  applySettings()
})

document.getElementById('settings-reset').addEventListener('click', function () {
  document.getElementById('settings-switch-system-theme').checked = true
  document.getElementById('settings-switch-hide-contents').checked = false
  document.getElementById('settings-switch-bw').checked = false
  document.getElementById('settings-select-font').value = 'noto'
  document.getElementById('settings-select-font-size').value = '1rem'
  document.getElementById('settings-select-line-height').value = '1.5'
})

document.getElementById('theme-switch-link').addEventListener('click', function () {
  if (settings.currentTheme === 'light') {
    settings.currentTheme = 'dark'
  } else {
    settings.currentTheme = 'light'
  }
  settings.systemTheme = false
  applyThemeFade(settings.currentTheme)
  saveSettings()
})

function applySettings() {
  if (settings.currentTheme === 'light') {
    switchToLight()
  } else {
    switchToDark()
  }

  if (settings.hideContents) {
    hideContents($('#btn-hide-contents'))
  } else {
    showContents($('#btn-hide-contents'))
  }

  if (settings.bw) {
    document.documentElement.classList.add('grayscale-filter')
  } else {
    document.documentElement.classList.remove('grayscale-filter')
  }

  if (settings.font === 'noto') {
    document.body.style.fontFamily = '"Noto Serif", serif';
  } else if (settings.font === 'atkinson') {
    document.body.style.fontFamily = '"Atkinson Hyperlegible", sans-serif';
  } else if (settings.font === 'garamond' ) {
    document.body.style.fontFamily = '"EB Garamond", serif';
  } else if (settings.font === 'stix' ) {
    document.body.style.fontFamily = '"STIX Two Text", serif';
  } else {
    document.body.style.fontFamily = '"Noto Serif", serif';
  }

  document.body.style.fontSize = settings.fontSize || '1rem'
  document.body.style.lineHeight = settings.lineHeight || '1.5'
}

function saveSettings() {
  localStorage.setItem('settings', JSON.stringify(settings))
}

// Theme switching
function isSystemDark() {
  return window.matchMedia('(prefers-color-scheme: dark)').matches
}

if (settings.systemTheme) {
  settings.currentTheme = isSystemDark() ? 'dark' : 'light'
}

window.matchMedia('(prefers-color-scheme: dark)').addEventListener('change', function (event) {
  if (settings.systemTheme) {
    settings.currentTheme = event.matches ? 'dark' : 'light'
    saveSettings()
    applySettings()
  }
})

pygments_themes = {
  "light": "css/highlight.css",
  "dark": "css/highlight-dark.css"
}
pygments_theme_link = $('#pygments-theme-link')

function switchToLight() {
  document.documentElement.setAttribute('data-bs-theme', 'light')
  pygments_theme_link.attr('href', pygments_themes['light'])
  $('img.woowoo-image, img.woowoo-tikz, span.mjx-mstyle').removeClass('fix-dark')
  $('#theme-switch-icon').removeClass('fa-sun')
  $('#theme-switch-icon').addClass('fa-moon')
}

function switchToDark() {
  $('img.woowoo-image, img.woowoo-tikz, span.mjx-mstyle').addClass('fix-dark')
  $('#theme-switch-icon').removeClass('fa-moon')
  $('#theme-switch-icon').addClass('fa-sun')
  document.documentElement.setAttribute('data-bs-theme', 'dark')
  pygments_theme_link.attr('href', pygments_themes['dark'])
  $('img.woowoo-image, img.woowoo-tikz, span.mjx-mstyle').addClass('fix-dark')
  $('#theme-switch-icon').removeClass('fa-moon')
  $('#theme-switch-icon').addClass('fa-sun')
}

function applyThemeFade(theme) {
  pygments_theme_link = $('#pygments-theme-link')

  if (theme === 'light') {
    $("html").fadeOut(1000, function () {
      // console.log('Going light...')
      document.documentElement.setAttribute('data-bs-theme', 'light')
      pygments_theme_link.attr('href', pygments_themes['light'])
      $('img.woowoo-image, img.woowoo-tikz, mjx-mstyle').removeClass('fix-dark')
      $('#theme-switch-icon').removeClass('fa-sun')
      $('#theme-switch-icon').addClass('fa-moon')
      $("html").fadeIn(1000)
    });
  } else {
    $("html").fadeOut(1000, function () {
      // console.log('Going dark...')
      document.documentElement.setAttribute('data-bs-theme', 'dark')
      pygments_theme_link.attr('href', pygments_themes['dark'])
      $('img.woowoo-image, img.woowoo-tikz, mjx-mstyle').addClass('fix-dark')
      $('#theme-switch-icon').removeClass('fa-moon')
      $('#theme-switch-icon').addClass('fa-sun')
      $("html").fadeIn(1000)
    });
  }
};

// Hide contents and navigation

function hideContents(button) {
  $('#contents').addClass('visually-hidden');
  $('#col-contents').removeClass('col-md-3').addClass('col-md-1');
  $('#col-body').removeClass('col-md-9').addClass('col-md-10');
  button.attr('data-contents', 'hidden');
  $('#icon-hide-contents').removeClass('fa-eye-slash').addClass('fa-eye');
};

function showContents(button) {
  $('#contents').removeClass('visually-hidden');
  $('#col-contents').removeClass('col-md-1').addClass('col-md-3');
  $('#col-body').removeClass('col-md-10').addClass('col-md-9');
  button.attr('data-contents', 'visible');
  $('#icon-hide-contents').removeClass('fa-eye').addClass('fa-eye-slash');
};

$('#btn-hide-contents').on('click', function () {
  the_button = $('#btn-hide-contents');
  state = the_button.attr('data-contents');

  if (state == 'visible') {
    hideContents(the_button)
  } else {
    showContents(the_button)
  };
});

$('#btn-scroll-up').on('click', function () {
  window.scrollTo({ top: 0, behavior: "smooth" });
});

$('#btn-scroll-down').on('click', function () {
  window.scrollTo({ top: document.body.scrollHeight, behavior: "smooth" });
});

// Geogebra

$('div.woowoo-geogebra').each(function () {
  material_id = $(this).attr('materialid');
  ggb_element_id = $(this).attr('id');
  params = {
    'material_id': material_id,
    'scaleContainerClass': 'text-center',
    'autoHeight': true,
    'preventFocus': true,
    'showZoomButtons': true,
    'showResetIcon': true,
    'showFullscreenButton': true,
    'borderRadius': 4
  };
  applet = new GGBApplet(params, true);
  window.addEventListener("load", function() {
    applet.inject(ggb_element_id);
  });
});

// render pseudocode; moved to _head, run once MathJax is ready!
// $('pre.pseudocode').each( function (index) {
//   pseudocode.renderElement($(this)[0], { lineNumber: true });
// });

saveSettings()
applySettings()
